#ifndef CONSOLHAWF3MATC_H
#define CONSOLHAWF3MATC_H

#include "genfile.h"
#include "lewis_schrefler.h"

class con_hawf3matc
{
 public:
  con_hawf3matc();    //constructor
  ~con_hawf3matc();   //destructor

  void read(XFILE *in);
  void print(FILE *out);

  void matcond_u (matrix &d,long ri,long ci,long ipp);
  void matcap_u (matrix &d,long ri,long ci,long ipp);
  void matcond1d_u (matrix &d,long ri,long ci,long ipp);
  void matcond2d_u (matrix &d,long ri,long ci,long ipp);
  void matcond2d_ax_u (matrix &d,long ri,long ci,long ipp);
  void matcond3d_u (matrix &d,long ri,long ci,long ipp);
  void matcap1d_u (matrix &d,long ri,long ci,long ipp);
  void matcap2d_u (matrix &d,long ri,long ci,long ipp);
  void matcap2d_ax_u (matrix &d,long ri,long ci,long ipp);
  void matcap3d_u (matrix &d,long ri,long ci,long ipp);

  void matcond_l (matrix &d,long ri,long ci,long ipp);
  void matcap_l (matrix &d,long ri,long ci,long ipp);
  void matcond1d_l (matrix &d,long ri,long ci,long ipp);
  void matcond2d_l (matrix &d,long ri,long ci,long ipp);
  void matcond2d_ax_l (matrix &d,long ri,long ci,long ipp);
  void matcond3d_l (matrix &d,long ri,long ci,long ipp);
  void matcap1d_l (matrix &d,long ri,long ci,long ipp);
  void matcap2d_l (matrix &d,long ri,long ci,long ipp);
  void matcap2d_ax_l (matrix &d,long ri,long ci,long ipp);
  void matcap3d_l (matrix &d,long ri,long ci,long ipp);

  void gaspress_check(double pw,double &pg,double t,long ipp);
  void waterpress_check(double &pw,double pg,double t,long ipp);

  void values_correction (vector &nv);

  double get_kuw(double pw, double pg, double t,long ipp);
  double get_kwu(double pw, double pg, double t,long ipp);

  double get_kug(double pw, double pg, double t,long ipp);
  double get_kgu(double pw, double pg, double t,long ipp);

  double get_capuw(double pw, double pg, double t,long ipp);
  double get_capwu(double pw, double pg, double t,long ipp);

  double get_capug(double pw, double pg, double t,long ipp);
  double get_capgu(double pw, double pg, double t,long ipp);

  double get_kut(double pw, double pg, double t,long ipp);
  double get_ktu(double pw, double pg, double t,long ipp);

  double get_caput(double pw, double pg, double t,long ipp);
  double get_captu(double pw, double pg, double t,long ipp);


  void rhs_u1 (matrix &d,long ri,long ci,long ipp);
  void rhs1d_u1 (matrix &d,long ri,long ci,long ipp);
  void rhs2d_u1 (matrix &d,long ri,long ci,long ipp);
  void rhs2d_ax_u1 (matrix &d,long ri,long ci,long ipp);
  void rhs3d_u1 (matrix &d,long ri,long ci,long ipp);

  double get_fuw1(double pw,double pg,double t,long ipp);
  double get_fug1(double pw,double pg,double t,long ipp);
  double get_fut1(double pw,double pg,double t,long ipp);

  void rhs_u2 (matrix &d,long ri,long ci,long ipp);
  void rhs1d_u2 (matrix &d,long ri,long ci,long ipp);
  void rhs2d_u2 (matrix &d,long ri,long ci,long ipp);
  void rhs2d_ax_u2 (matrix &d,long ri,long ci,long ipp);
  void rhs3d_u2 (matrix &d,long ri,long ci,long ipp);

  double get_fut2(double pw,double pg,double t,long ipp);

  double get_sw(double pw, double pg, double t,long ipp);
  double get_alpha(double pw, double pg, double t,long ipp);

  double get_rhogw(double pw,double pg,double t);
  double get_pgw(double pw,double pg,double t);
  double get_pgws(double t);
  double get_rhow(double t);
  double get_rhos(double t);
  double get_porosity(double pw, double pg,double t,long ipp);
  double get_rhoga(double pw,double pg,double t);
  double get_rhog(double pw,double pg,double t);
  double get_betas(long ipp);
  double get_dhvap(double t);
  double get_e(long ipp);
  double get_nu(long ipp);

  /// Lewis and Schrefler's retention curve:
  lewis_reten lewis_ret;

 private:
  

  heatairwaterflowmechtype model_type;
  int compress;
  
  double ma;//molar mass of dry air
  double mw;//molar mass of water
  double gasr;//universal gas constant

  double t0;//[K] reference temperature 
  double p0;//Pa reference athmospheric pressure

  // PHYSICAL PROPERTIES OF DRY AIR
  double muga0,alphaa,betaa;

  // PHYSICAL PROPERTIES OF WATER VAPOUR
  // from D.Gawin, F.Pesavento (PRVAP.f90)
  double dv0,bv;
  double c8,c9,c10,c11,c12,c13;
  double mugw0,alphaw;

  // PHYSICAL PROPERTIES OF WATER
  // from Dariusz Gawin (WATPROP.f90)
  double rhow0;//density at refernce temperature and pressure
  double tcr;//[K] critical temperature of water
  double cwat;
  double betawat;
  double hvap0;//latent heat of vaporization at reference temperature
  double a0,a1,a2;
  double a3,a4,a5;
  double b0,b1,b2;
  double b3,b4,b5;
  double pr1,prif;
  double muw0;//reference dynamic viscosity of water
  double conb;
  double conc;
  double cpw;//water specific heat at const. pressure = 4181 J/kg.K
  double lambdaw;//water heat conductivity = 0.6 W/(m.K)
  double kw0;//compresibility coefficient of water

  //PHYSICAL PROPERTIES OF SOIL
  double alpha0; //initial Boit's constant
  double ks0;    //inital bulk modulus of solid phase
  double kt0;    //inital bulk modulus of porous medium
  double phi0;   //inital porosity
  double kintr0; //intial intrinsic permeability
  double betas0; //inital cubic thermal dilatation coefficient
  double deff0;  //initial effective diffusivity
  double rhos0;  //inital volume density of soil skeleton
  double cps0;   //inital specific heat of soil skeleton
  double lambda0;//effective thermal conductivity of soil skeleton
  double lambda_dry;//effective thermal conductivity of dry soil
  double lambda_wet;//effective thermal conductivity of saturated soil
  double sirr0; //irreversible (residual) saturation degree
  double tau0; //tortuosity factor 0.4 to 0.6

  double emod,nu;

  double mefel_units; //basic units for pressures = Pa (Pascals)

};  

#endif
